### R code from vignette source '04.Rnw'

###################################################
### code chunk number 1: c0
###################################################
suppressPackageStartupMessages(require(SOAR))
options(width=75)
graphics.off()
pdf(file="Fig/04_All.pdf", onefile=TRUE)
if(length(ls() > 0)) Store(ls())
options(continue="  ",
        SweaveHooks = list(fig = function() par(las = 1,
        family = "mono")))
## Sys.setenv(R_LOCAL_CACHE=".R_Store")
Attach()


###################################################
### code chunk number 2: 04.Rnw:77-80 (eval = FALSE)
###################################################
## library(MASS)
## ?petrol
## head(petrol)


###################################################
### code chunk number 3: 04.Rnw:83-85
###################################################
library(MASS)
head(petrol)


###################################################
### code chunk number 4: c0
###################################################
require(lattice)
## lattice.options(default.theme = standard.theme(color=TRUE))
print(xyplot(Y ~ EP | No, petrol, as.table = TRUE, aspect=1,
             panel = function(x, y, ...) {
               panel.xyplot(x, y, ...)
               panel.lmline(x, y)
             }))

petrol <- within(petrol, EPc <- EP - mean(EP))  ### for convenience
Store(petrol)


###################################################
### code chunk number 5: c1
###################################################
options(show.signif.stars = FALSE)
m3 <- lm(Y ~ 0 + No/EP, petrol)            ## 10 ints + 10 slopes
m2 <- lm(Y ~ 0 + No+EP, petrol)            ## 10 ints + 1 slope
m1 <- lm(Y ~ 1 + SG+VP+V10+EP, petrol)     ## (1 int + 3 coeffs) + 1 slope
anova(m1, m2, m3)


###################################################
### code chunk number 6: 04.Rnw:136-138
###################################################
library(SOAR)
Attach()


###################################################
### code chunk number 7: c2
###################################################
suppressPackageStartupMessages(library(lme4))  ## alt. nlme
Rm1 <- lmer(Y ~ 1 + SG+VP+V10 + EPc + (1|No),     data = petrol)
Rm2 <- lmer(Y ~ 1 + SG+VP+V10 + EPc + (1+EPc|No), data = petrol)
anova(Rm1, Rm2)


###################################################
### code chunk number 8: c2a
###################################################
Rm1_ML <- update(Rm1, REML = FALSE)
Rm2_ML <- update(Rm2, REML = FALSE)
anova(Rm1_ML, Rm2_ML)


###################################################
### code chunk number 9: c3
###################################################
print(summary(Rm1), correlation = FALSE)


###################################################
### code chunk number 10: c3a
###################################################
print(summary(Rm2), correlation = FALSE)


###################################################
### code chunk number 11: c4
###################################################
cbind(Rm1 = ranef(Rm1)$No, Rm2 = ranef(Rm2)$No)


###################################################
### code chunk number 12: c5
###################################################
VarCorr(Rm2)


###################################################
### code chunk number 13: c6
###################################################
data(Headrope)
dim(Headrope)
head(Headrope, 2)
Headrope <- within(Headrope, YearF <- factor(YearF)) ## needed
Store(Headrope)


###################################################
### code chunk number 14: c7
###################################################
HRmodel1 <- lmer(log(Catch) ~ 0 + log(Days) + Y2K + log(Head) +
                 log(Power) + log(Hull) + Stock +
                 (1|Vessel) + (1|YearF/Stock), Headrope)
HRmodel1_ML <- update(HRmodel1, REML = FALSE)
Store(HRmodel1)


###################################################
### code chunk number 15: c8 (eval = FALSE)
###################################################
## HRmodel2 <- lmer(log(Catch) ~ 0 + log(Days) + Y2K + log(Head) +
##                  log(Power) + log(Hull) + Stock +
##                  (1|Vessel) + (0+Stock|YearF),
##                  data = Headrope)
## HRmodel2_ML <- update(HRmodel2, REML = FALSE)
## Store(HRmodel2, HRmodel2_ML)


###################################################
### code chunk number 16: c10
###################################################
anova(HRmodel1_ML, HRmodel2_ML)


###################################################
### code chunk number 17: c9
###################################################
cbind(m1 = fixef(HRmodel1), m2 = fixef(HRmodel2))


###################################################
### code chunk number 18: c10
###################################################
print(summary(HRmodel2), correlation = FALSE)


###################################################
### code chunk number 19: g1 (eval = FALSE)
###################################################
## library(splines)
## library(MASS)
## TModelGLMM <- glmmPQL(Psem/Total ~ ns(Coast, 6) + ns(Sea, 5) +
##                       twoWay(DayOfYear, Sea) + ns(Depth, k=5) +
##                       Hyear(DayOfYear, 2) + ns(Mud, k=5),
##                       random = ~1|Survey,
##                       family = quasibinomial, data= Tigers,
##                       niter = 40, weights = Total)
## Store(TModelGLMM)


###################################################
### code chunk number 20: g2 (eval = FALSE)
###################################################
## library(mgcv)
## TModelGAMM <- gamm(formula = Psem/Total ~ s(Coast, k=5) + s(Sea,k=5) +
##                    twoWay(DayOfYear, Sea) +
##                    s(DayOfYear, k=5, bs="cc") + s(Depth,k=5) +
##                    s(Mud, k=5),
##                    random = list(Survey = ~1),
##                    family = quasibinomial, data = Tigers,
##                    niterPQL = 40,
##                    weights = Total)
## Store(TModelGAMM)


###################################################
### code chunk number 21: g3
###################################################
re1 <- ranef(TModelGLMM)
re2 <- ranef(TModelGAMM$lme)$Survey  ## obscure
re12 <- cbind(re1, re2)
names(re12) <- c("GLMM", "GAMM")
re12


###################################################
### code chunk number 22: g4
###################################################
layout(rbind(1:2), widths = c(3.5,1))
library(plotrix)
with(re12, {
  nos <- seq_along(GLMM)
  plot(GLMM ~ GAMM, pch = 20,col="red", bty="n")
  pos <- thigmophobe(GAMM, GLMM)
  text(GLMM ~ GAMM, labels = nos, xpd = NA, pos=pos)
  par(mar = c(0,0,0,0), xpd = NA)
  frame()
  legend("center", paste(format(nos), rownames(re12),
                         sep = ": "), bty="n")
})


###################################################
### code chunk number 23: g4
###################################################
Harm <- function (theta, k = 4) {
  X <- matrix(0, length(theta), 2 * k)
  nam <- as.vector(outer(c("c", "s"), 1:k, paste, sep = ""))
  dimnames(X) <- list(names(theta), nam)
  m <- 0
  for (j in 1:k) {
    X[, (m <- m + 1)] <- cos(j * theta)
    X[, (m <- m + 1)] <- sin(j * theta)
  }
  X
}

Hyear <- function(x, k = 4)
    Harm(2*base::pi*x/365.25, k)


twoWay <- local({

  `%star%` <- function(X, Y) {
    X <- as.matrix(X)
    Y <- as.matrix(Y)
    stopifnot(is.numeric(X), is.numeric(Y),
              nrow(X) == nrow(Y))
    XY <- matrix(NA, nrow(X), ncol(X)*ncol(Y))
    k <- 0
    for(i in 1:ncol(X))
        for(j in 1:ncol(Y)) {
          k <- k+1
          XY[, k] <- X[, i] * Y[, j]
        }
    XY
  }

  function(day, sea, k = c(3,2))
      Hyear(day, k[1]) %star% ns(sea, k[2])
})


###################################################
### code chunk number 24: 04.Rnw:475-477
###################################################
toLatex(sessionInfo())
graphics.off()


