### R code from vignette source '05.Rnw'

###################################################
### code chunk number 1: c0
###################################################
suppressPackageStartupMessages(require(SOAR))
options(width=75)
graphics.off()
pdf(file="Fig/05_All.pdf", onefile=TRUE)
if(length(ls() > 0)) Store(ls())
options(continue="  ",
        SweaveHooks = list(fig = function() par(las = 1, xpd = NA,
        family = "mono")))
## Sys.setenv(R_LOCAL_CACHE=".R_Store")
Attach()


###################################################
### code chunk number 2: janka
###################################################
if(require(tree)) {
  janka.tm <- tree(Hardness ~ Density, janka)
  plot(janka.tm); text(janka.tm)
}


###################################################
### code chunk number 3: 05.Rnw:108-112
###################################################
if(require(tree)) {
  partition.tree(janka.tm)
  points(Hardness ~ Density, janka, col = "brown")
}


###################################################
### code chunk number 4: 05.Rnw:119-123
###################################################
require(rpart)
janka.rm <- rpart(Hardness ~ Density, janka,
   control = rpart.control(cp = 0.001, minsize = 3))
plot(janka.rm); text(janka.rm, xpd = NA)


###################################################
### code chunk number 5: 05.Rnw:130-131
###################################################
plotcp(janka.rm)


###################################################
### code chunk number 6: 05.Rnw:138-140
###################################################
janka.rmp <- prune(janka.rm, cp = oneSERule(janka.rm))
plot(janka.rmp); text(janka.rmp)


###################################################
### code chunk number 7: t1
###################################################
data(creditCards)
dim(creditCards)
Store(creditCards)


###################################################
### code chunk number 8: t2
###################################################
set.seed(1234)
nCC <- nrow(creditCards)
train <- sample(nCC, 1000)
CCTrain <- creditCards[train, ]
CCTest <- creditCards[-train, ]
Store(CCTrain, CCTest)  ## for safe keeping


###################################################
### code chunk number 9: t3
###################################################
library(rpart)
CCTree <- rpart(credit.card.owner ~ ., CCTrain)
plot(CCTree)
text(CCTree)
Store(CCTree)


###################################################
### code chunk number 10: t4
###################################################
plotcp(CCTree)


###################################################
### code chunk number 11: t5
###################################################
CCPTree <- prune(CCTree, cp = oneSERule(CCTree))
plot(CCPTree)
text(CCPTree)
Store(CCPTree)


###################################################
### code chunk number 12: oneSE
###################################################
oneSERule <- function (tree, f, ...)
UseMethod("oneSERule")

oneSERule.rpart <- function (tree, f = 1, ...) {
    cp <- data.frame(tree$cptable)    #$
    imin <- with(cp, which(xerror == min(xerror))[1])
    with(cp, CP[which(xerror <= xerror[imin] + f * xstd[imin])[1]])
}
Store(oneSERule, oneSERule.rpart)  ## to make available later


###################################################
### code chunk number 13: tbag
###################################################
bagRpart <- local({
  bsample <- function(dataFrame) # bootstrap sampling
    dataFrame[sample(nrow(dataFrame), rep = TRUE),  ]
  function(object, data = eval.parent(object$call$data),
           nBags=200, type = c("standard", "bayesian"), ...) {
    type <- match.arg(type)
    bagsFull <- vector("list", nBags)
    if(type == "standard") {
      for(j in 1:nBags)
          bagsFull[[j]] <- update(object, data = bsample(data))
      } else {
        nCases <- nrow(data)
        for(j in 1:nBags)
            bagsFull[[j]] <- update(object, weights = rexp(nCases))
        }
    class(bagsFull) <- "bagRpart"
    bagsFull
  }
})

## a prediction method for the objects (somewhat tricky!)
predict.bagRpart <- function(object, newdata, ...) {
  X <- sapply(object, predict, newdata = newdata, type = "class")
  candidates <- levels(predict(object[[1]], type = "class"))
  X <- t(apply(X, 1, function(r) table(factor(r, levels = candidates))))
  factor(candidates[max.col(X)], levels = candidates)
}
Store(bagRpart, predict.bagRpart)


###################################################
### code chunk number 14: tbag2
###################################################
if(!exists("CCSBag")) {
  set.seed(4321)
  Obj <- update(CCTree, cp = 0.005, minsplit = 9)  ## expand the tree
  CCSBag <- bagRpart(Obj, nBags = 100)
  CCBBag <- bagRpart(Obj, nBags = 100, type = "bayes")
  rm(Obj)
  Store(CCSBag, CCBBag)
}


###################################################
### code chunk number 15: trf1
###################################################
n <- nrow(CCTest)
X <- replicate(100,
       table(factor(sample(n, rep=TRUE), levels = 1:n)))
(lims <- range(rowSums(X > 0)))
rm(n, X)


###################################################
### code chunk number 16: trf2
###################################################
suppressPackageStartupMessages(library(randomForest))
(CCRf <- randomForest(credit.card.owner ~ ., CCTrain, ntree = 100))
Store(CCRf)


###################################################
### code chunk number 17: trf3
###################################################
v <- varImpPlot(CCRf)  ## causes a plot
v <- sort(drop(v), decreasing = TRUE)
v[1:6]
bestFew <- setdiff(names(v)[1:20], "current.profession")  ## used later


###################################################
### code chunk number 18: g1
###################################################
form <- as.formula(paste("credit.card.owner~", paste(bestFew, collapse="+")))

Call <- substitute(glm(FORM, binomial, CCTrain), list(FORM = form))
CCGlmNaive <- eval(Call)
Store(CCGlmNaive)

if(!exists("CCGlmAIC")) {
  upp <- paste("~", paste(setdiff(names(CCTrain), "credit.card.owner"),
               collapse="+"))
  upp <- as.formula(upp)
  start <- glm(credit.card.owner ~ mean.check.credits+gender,
               binomial, CCTrain)
  CCGlmAIC <- stepAIC(start, list(upper=upp, low=~1), trace=FALSE)
  CCGlmBIC <- stepAIC(CCGlmAIC, trace = FALSE, k = log(nrow(CCTrain)))
  Store(CCGlmAIC, CCGlmBIC)
  rm(start, upp)
}


###################################################
### code chunk number 19: Class
###################################################
Class <- function(object, newdata, ...)
    UseMethod("Class")

Class.rpart <- function(object, newdata, ...)
    predict(object, newdata, type = "class")

Class.bagRpart <- function(object, newdata, ...)
    predict(object, newdata)

Class.randomForest <- predict

Class.glm <- function(object, newdata, ...) {
  ## only applies for binomial glms and symmetric link fns
  predict(object,newdata) > 0
}


###################################################
### code chunk number 20: errors
###################################################
errorRate <- function(tab) 100*(1 - sum(diag(tab))/sum(tab))
true <- CCTest$credit.card.owner  #$
sort(sapply(list(Tree = CCTree,
                 Pruned = CCPTree,
                 Bagging = CCSBag,
                 Bayes = CCBBag,
                 RandomF = CCRf,
                 NaiveGLM = CCGlmNaive,
                 Glm_AIC = CCGlmAIC,
                 Glm_BIC = CCGlmBIC),
             function(x) errorRate(table(Class(x, CCTest),
                                         true))))


###################################################
### code chunk number 21: 05.Rnw:434-436
###################################################
toLatex(sessionInfo())
graphics.off()


